from typing import List, Optional

from llama_index.core.readers.base import BaseReader
from llama_index.core.schema import Document

from llama_index.readers.gitbook.gitbook_client import GitbookClient

VALID_METADATA_FIELDS = {"path", "title", "description", "parent"}


class SimpleGitbookReader(BaseReader):
    """
    Simple gitbook reader.

    Convert each gitbook page into Document used by LlamaIndex.

    Args:
        api_token (str): Gitbook API Token.
        api_url (str): Gitbook API Endpoint.

    """

    def __init__(self, api_token: str, api_url: str = None) -> None:
        """Initialize with parameters."""
        self.client = GitbookClient(api_token, api_url)

    def load_data(
        self,
        space_id: str,
        metadata_names: Optional[List[str]] = None,
        show_progress=False,
    ) -> List[Document]:
        """
        Load data from the input directory.

        Args:
            space_id (str): Gitbook space id
            metadata_names (Optional[List[str]]): names of the fields to be added
                to the metadata attribute of the Document.
                only 'path', 'title', 'description', 'parent' are available
                Defaults to None
            show_progress (bool, optional): Show progress bar. Defaults to False

        Returns:
            List[Document]: A list of documents.

        """
        if metadata_names:
            invalid_fields = set(metadata_names) - VALID_METADATA_FIELDS
            if invalid_fields:
                raise ValueError(
                    f"Invalid metadata fields: {', '.join(invalid_fields)}"
                )

        documents = []
        pages = self.client.list_pages(space_id)

        if show_progress:
            from tqdm import tqdm

            iterator = tqdm(pages, desc="Downloading pages")
        else:
            iterator = pages

        for page in iterator:
            id = page.get("id")
            content = self.client.get_page_markdown(space_id, id)
            if not content:
                print(f"Warning: No content found for page ID {id}. Skipping...")
                continue

            if metadata_names is None:
                documents.append(
                    Document(text=content, id_=id, metadata={"path": page.get("path")})
                )
            else:
                try:
                    metadata = {name: page.get(name) for name in metadata_names}
                except KeyError as err:
                    raise ValueError(
                        f"{err.args[0]} field is not available. Choose from {', '.join(VALID_METADATA_FIELDS)}"
                    ) from err
                documents.append(Document(text=content, id_=id, metadata=metadata))

        return documents


if __name__ == "__main__":
    import os
    import sys

    def load_env_file():
        """Load environment variables from .env file."""
        current_dir = os.path.dirname(os.path.abspath(__file__))
        env_path = os.path.join(current_dir, "../../../.env")
        if os.path.exists(env_path):
            with open(env_path) as f:
                for line in f:
                    line = line.strip()
                    if line and not line.startswith("#"):
                        key, value = line.split("=", 1)
                        os.environ[key.strip()] = value.strip()

    load_env_file()
    api_token = os.getenv("GITBOOK_API_TOKEN")
    space_id = os.getenv("GITBOOK_SPACE_ID")

    if not api_token or not space_id:
        print("Error: GITBOOK_API_TOKEN and GITBOOK_SPACE_ID must be set in .env file")
        sys.exit(1)

    reader = SimpleGitbookReader(api_token)
    print(reader.load_data(space_id, show_progress=True))



import requests
from typing import List, Dict

DEFAULT_GITBOOK_API_URL = "https://api.gitbook.com/v1"


class GitbookClient:
    """
    Gitbook Restful API Client.

    Helper Class to invoke gitbook restful api & parse result

    Args:
        api_token (str): Gitbook API Token.
        api_url (str): Gitbook API Endpoint.

    """

    def __init__(self, api_token: str, api_url: str = DEFAULT_GITBOOK_API_URL):
        self.api_token = api_token
        self.base_url = api_url or DEFAULT_GITBOOK_API_URL
        self.headers = {
            "Authorization": f"Bearer {self.api_token}",
            "Content-Type": "application/json",
        }

    def _make_request(self, url: str) -> Dict:
        """Helper method to handle common HTTP GET requests."""
        try:
            response = requests.get(url, headers=self.headers)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as e:
            return self._handle_error(e)

    def get_space(self, space_id) -> Dict:
        """Gets information for a specific space."""
        url = f"{self.base_url}/spaces/{space_id}"
        return self._make_request(url)

    def list_pages(self, space_id) -> List[Dict]:
        """Gets all pages in a specific space."""
        space_info = self.get_space(space_id)
        url = f"{self.base_url}/spaces/{space_id}/content"
        space = self._make_request(url)

        pages_info = []
        for page in space.get("pages"):
            GitbookClient._extract_page_info(
                pages_info, page, space_info.get("title", "ROOT")
            )
        return pages_info

    def get_page(self, space_id, page_id) -> Dict:
        """Gets the details of a specific page."""
        url = (
            f"{self.base_url}/spaces/{space_id}/content/page/{page_id}?format=markdown"
        )
        return self._make_request(url)

    def get_page_markdown(self, space_id, page_id) -> str:
        """Gets the content of a specific page in Markdown format."""
        page_content = self.get_page(space_id, page_id)
        return page_content.get("markdown")

    def _handle_error(self, response):
        """Handles HTTP errors."""
        if isinstance(response, requests.exceptions.HTTPError):
            error_message = f"Error: {response.response.status_code} Client Error: {response.response.reason}"
        else:
            error_message = f"Error: {response}"
        raise Exception(error_message)

    @staticmethod
    def _extract_page_info(
        pages: list, page: dict, prev_title: str = "", parent: str = ""
    ):
        pageType = page.get("type", "")
        title = prev_title + " > " + page.get("title")
        id = page.get("id")
        if pageType == "document":
            pages.append(
                {
                    "id": id,
                    "title": title,
                    "path": page.get("path"),
                    "description": page.get("description", ""),
                    "parent": parent,
                }
            )
            for _page in page.get("pages"):
                GitbookClient._extract_page_info(pages, _page, title, id)
        elif pageType == "group":
            for _page in page.get("pages"):
                GitbookClient._extract_page_info(pages, _page, title, id)
